#!/usr/bin/env python
'''
GDS AddFile is a simple tool, to manually add text files and source code to your
Google Desktop Search index - like favorites which should be in the indes.

You are also tired of a huge index and not finding what you search for, you can
reduce the amount of places to be indexed and collect with this tools your high-
lights to the index. With this you will only find what you are looking for and
what you have marked to be indexed.

INSTALLATION:

Simply call the GDSAddFile.exe, which will add a menu entry to your context menu
and register GDS AddFile as a component to Google Desktop Search. That`s all.

UNINSTALLATION:

If you want to uninstall this tool, call it with -u as the argument:

GDSAddFile.exe -u

Dipl.-Ing. Mustafa Goermezer
mustafa@goermezer.de
http://www.goermezer.de

'''

import sys
import win32com.client
import time
import pywintypes
import pythoncom
import win32con
import win32api
import win32gui
import optparse
import os

_GUID = '{6739DD56-05B9-4220-AC1D-00F04EF30D98}'

""" misc utilities """
def GetPythonwExePath():
    """ Get path to current version of pythonw.exe """
    pythonExePath = ""
    try:
        pythonwExeName = "pythonw.exe"
        pythonInstallHiveKey = win32con.HKEY_LOCAL_MACHINE
        pythonInstallKey = r"Software\Python\PythonCore\%s\InstallPath" % sys.winver
        pythonInstallDir, typeId = ReadRegistryValue(pythonInstallHiveKey, pythonInstallKey, "")
        pythonwExePath = os.path.join(pythonInstallDir, pythonwExeName)
    except Exception, e:
        print "GetPythonExePath failed:", e
    return pythonwExePath

def GetModulePath():
    """ Get path to this module """
    return GetModulePath.func_code.co_filename

def WriteRegistryValue(hiveKey, key, name, typeId, data):
    """ Simple api to write one value to Windows registry.
    If 'name' is empty string, writes to default value."""
    try:
        keyHandle = OpenRegistryKey(hiveKey, key)
        win32api.RegSetValueEx(keyHandle, name, 0, typeId, data)
        win32api.RegCloseKey(keyHandle)
    except Exception, e:
        print "WriteRegistry failed:", e

def DeleteRegistryKey(hiveKey, key, name):
    """ Simple api to delete one value from Windows registry."""

    try:
        keyHandle = OpenRegistryKey(hiveKey, key)
        win32api.RegDeleteKey(keyHandle, name)
        #win32api.RegCloseKey(keyHandle)
    except Exception, e:
        print "DeleteRegistryKey failed:", e


""" Windows Registry utilities """
def OpenRegistryKey(hiveKey, key):
    keyHandle = None
    try:
        curKey = ""
        keyItems = key.split('\\')
        for keyItem in keyItems:
            if curKey:
                curKey = curKey + "\\" + keyItem
            else:
                curKey = keyItem
            keyHandle = win32api.RegCreateKey(hiveKey, curKey)
    except Exception, e:
        keyHandle = None
        print "OpenRegistryKey failed:", e
    return keyHandle

def ReadRegistryValue(hiveKey, key, name):
    """ Simple api to read one value from Windows registry.
    If 'name' is empty string, reads default value."""
    data = typeId = None
    try:
        hKey = win32api.RegOpenKeyEx(hiveKey, key, 0, win32con.KEY_ALL_ACCESS)
        data, typeId = win32api.RegQueryValueEx(hKey, name)
        win32api.RegCloseKey(hKey)
    except Exception, e:
        print "ReadRegistryValue failed:", e
    return data, typeId

def WriteRegistryValue(hiveKey, key, name, typeId, data):
    """ Simple api to write one value to Windows registry.
    If 'name' is empty string, writes to default value."""
    try:
        keyHandle = OpenRegistryKey(hiveKey, key)
        win32api.RegSetValue(keyHandle, name, typeId, data)
        win32api.RegCloseKey(keyHandle)
    except Exception, e:
        #print "WriteRegistry failed:", e
        win32gui.MessageBox(0, "WriteRegistry failed:", e , "GDSAddFile Error" , 0 )

def WriteLastTime():
    secsString = str(time.time())
    WriteRegistryValue(win32con.HKEY_CLASSES_ROOT, r"*\shell\GDS AddFile\time", "", win32con.REG_SZ, secsString)

def we_are_frozen():
    """Returns whether we are frozen via py2exe.
    This will affect how we find out where we are located."""

    return hasattr(sys, "frozen")

def module_path():
    """ This will get the program's directory,
    even if we are frozen using py2exe"""

    if we_are_frozen():
        return os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding( )))
    return os.path.dirname(unicode(__file__, sys.getfilesystemencoding( )))


def InstallGDSAdder():
    """ Installs GDS AddFile to the Windows registry """
    if hasattr(sys, "frozen") == True:
        command = '"%s" "%s"'%(os.path.join(module_path(), os.path.basename(sys.argv[0])), "%1")
        #win32gui.MessageBox(0, "%s"%command , "GDSAddFile" , 0 )

    else:
        command = '"%s" "%s" "%s"' % (GetPythonwExePath(), GetModulePath(), "%1")
    WriteRegistryValue(win32con.HKEY_CLASSES_ROOT, r"*\shell\GDS AddFile\Command", "", win32con.REG_SZ, command)
    WriteRegistryValue(win32con.HKEY_CLASSES_ROOT, r"Folder\shell\GDS AddFile\Command", "", win32con.REG_SZ, command)
    WriteLastTime()

def Main(addfile):
  '''This function contains the logic for the command-line UI for GDS AddFile.'''

  # Set up options and parse arguments.
  parser = optparse.OptionParser(usage='%prog [-d to uninstall] or [FILENAME to add to index]')
  parser.add_option('-u', '--unregister', action='store_true', dest='unreg',
                    help='Run with this flag to unregister the plugin. '
                         'All other options are ignored when you use this flag.')
  (options, args) = parser.parse_args()
  #if len(args) < 1 and not options.unreg:
  #  parser.error('Provide at least one Argument')

  try:
    obj = win32com.client.Dispatch('GoogleDesktopSearch.Register')
  except pythoncom.ole_error:
    print ('ERROR: You need to install Google Desktop Search to be able to '
           'use GDS AddFile.')
    sys.exit(2)

  if len(args) < 2 and not options.unreg:

    try:
      # Register with GDS.  This is a one-time operation and will return an
      # error if already registered.  We cheat and just catch the error and
      # do nothing.

      # We try two different methods since different versions of GD have
      # different names for the registration method.  We ignore the specific
      # exception that we get if the method is not supported.
      try:
        obj = win32com.client.Dispatch('GoogleDesktopSearch.Register')
        obj.RegisterComponent(_GUID,
                 ['Title', 'GDS AddFile', 'Description', 'A simple context menu entry for text and source code '
                  'that lets you manually add them to your Google Desktop Search index.', 'Icon', '%SystemRoot%\system32\SHELL32.dll,134'])
        print 'Registered via RegisterComponent.'
      except AttributeError, e:
        #if (len(e.args) == 0 or e.args[0] != 'GoogleDesktopSearch.Register.RegisterComponent'):
        #  raise e
        #else:
        obj.RegisterIndexingComponent(_GUID,
             ['Title', 'GDS AddFile', 'Description', 'A simple context menu entry for text and source code '
              'that lets you manually add them to your Google Desktop Search index.', 'Icon', '%SystemRoot%\system32\SHELL32.dll,134'])
        print 'Registered via RegisterIndexingComponent.'
    except pywintypes.com_error, e:
      if len(e.args) > 0 and e.args[0] == -2147352567:
        # This is the error we get if already registered.
        pass
      else:
        raise e


    filehandle = open(addfile)
    content = filehandle.read()
    filehandle.close()
    filestat=os.stat(addfile)
    event_factory = win32com.client.Dispatch('GoogleDesktopSearch.EventFactory')
    event = event_factory.CreateEvent(_GUID, 'Google.Desktop.File')
    event.AddProperty('format', 'text/plain')
    event.AddProperty('content', str(content))
    event.AddProperty('uri', addfile)
    #event.AddProperty('last_modified_time', pywintypes.Time(time.time() + time.timezone))
    event.AddProperty('last_modified_time', pywintypes.Time(filestat.st_atime + time.timezone))
    event.AddProperty('title', os.path.basename(addfile))
    try:
        event.Send(0x01)
    except:
        win32gui.MessageBox(0, "Could not add file to Google Desktop Index. Is Google Desktop Search running ?" , "GDSAddFile: Could not add file to index!" , 0 )


  else:
      #obj = win32com.client.gencache.EnsureDispatch('GoogleDesktopSearch.Register')
      #obj2.UnregisterIndexingComponent(_GUID)
      #print 'googd'
      # Try both approaches to unregister, too.
      try:
        obj.UnregisterIndexingComponent(_GUID)
        print 'Unregistered IndexingComponent'
      except:
        try:
          #print 'Unregistering Component'
          obj.UnregisterComponent(_GUID)
          print 'Unregistered Component'
        except:
          try:
            DeleteRegistryKey(win32con.HKEY_CLASSES_ROOT, r"*\shell\GDS AddFile\time", "")
            DeleteRegistryKey(win32con.HKEY_CLASSES_ROOT, r"*\shell\GDS AddFile\command", "")
            DeleteRegistryKey(win32con.HKEY_CLASSES_ROOT, r"*\shell\GDS AddFile", "")
            DeleteRegistryKey(win32con.HKEY_CLASSES_ROOT, r"Folder\shell\GDS AddFile\Command", "")
            DeleteRegistryKey(win32con.HKEY_CLASSES_ROOT, r"Folder\shell\GDS AddFile", "")
            win32gui.MessageBox(0, "GDS AddFile was successfully uninstalled now. Thanks for using it!\n\nDipl.-Ing. Mustafa Goermezer\nmustafa@goermezer.de\nhttp://www.goermezer.de" , "GDS AddFile uninstalled" , 0 )
          except:
            win32gui.MessageBox(0, "Could not uninstall GDS AddFile! Sorry for that" , "GDS AddFile not uninstalled" , 0 )
  #sys.exit(0)

if __name__ == "__main__":
    if len(sys.argv) > 1:
        """ If invoked through a right-click, there will be a path argument """
        path = sys.argv[1]
        Main(os.path.abspath(path))

    else:
        """ If module is run by itself, install GDS AddFile to registry """
        InstallGDSAdder()
        win32gui.MessageBox(0, "GDSAddFile installed. Now you can use it via rightclick in your file browser whenever you want\nto add a text file or source code exclusively to your Google Desktop Search Index. You can\nremove this tool with GDSAddFile.exe -u" , "GDS AddFile installed" , 0 )
